//
// Enum.h
//
// $Id: //poco/1.4/CppParser/include/Poco/CppParser/Enum.h#1 $
//
// Library: CppParser
// Package: SymbolTable
// Module:  Enum
//
// Definition of the Enum class.
//
// Copyright (c) 2006, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef CppParser_Enum_INCLUDED
#define CppParser_Enum_INCLUDED


#include "Poco/CppParser/CppParser.h"
#include "Poco/CppParser/Symbol.h"
#include <vector>


namespace Poco {
namespace CppParser {


class EnumValue;


class CppParser_API Enum: public Symbol
	/// This class represents an enum declaration.
	///
	/// An enumeration has an optional name and
	/// a collection of EnumValues.
{
public:
	typedef std::vector<EnumValue*> Values;
	typedef Values::const_iterator Iterator;

	Enum(const std::string& name, NameSpace* pNameSpace);
		/// Creates the Enum.
		///
		/// If name is the empty string, an internal name
		/// in the form #AnonEnum<n> (where <n> is a unique integer)
		/// will be assigned.

	~Enum();
		/// Destroys the Enum.

	void addValue(EnumValue* pValue);
		/// Adds an enum value. The Enum takes ownership of the value.
		
	Iterator begin() const;
		/// Returns an iterator for iterating over the Enum's EnumValue's.
		
	Iterator end() const;
		/// Returns an iterator for iterating over the Enum's EnumValue's.
	
	Symbol::Kind kind() const;
	std::string toString() const;

protected:
	static std::string processName(const std::string& name);

private:	
	Values _values;
	static int _count;
};


} } // namespace Poco::CppParser


#endif // CppParser_Enum_INCLUDED
