//
// Stringifier.h
//
// $Id$
//
// Library: JSON
// Package: JSON
// Module:  Stringifier
//
// Definition of the Stringifier class.
//
// Copyright (c) 2012, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
//
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef JSON_JSONStringifier_INCLUDED
#define JSON_JSONStringifier_INCLUDED


#include "Poco/Dynamic/Var.h"
#include "Poco/JSON/JSON.h"
#include <ostream>


namespace Poco {
namespace JSON {


class JSON_API Stringifier
	/// Helper class for creating a String from a JSON object or array
{
public:
	static void condense(const Dynamic::Var& any, std::ostream& out);
		/// Writes a condensed string representation of the value to the output stream while preserving the insertion order.
		/// This is just a "shortcut" to stringify(any, out) with name indicating the function effect.

	static void stringify(const Dynamic::Var& any, bool preserveInsertionOrder, std::ostream& out, unsigned int indent = 0);
		/// Writes a String representation of the value to the output stream while preserving the insertion order.
		/// When indent is 0, the generated string will be created as small as possible (condensed).
		/// When preserveInsertionOrder is true, the original string object members order will be preserved.
		/// This is a "shortcut" to stringify(any, out, indent, -1, preserveInsertionOrder).

	static void stringify(const Dynamic::Var& any, std::ostream& out, unsigned int indent = 0, int step = -1, bool preserveInsertionOrder = false);
		/// Writes a String representation of the value to the output stream.
		/// When indent is 0, the String will be created as small as possible.
		/// When preserveInsertionOrder is true, the original string object members order will be preserved;
		/// otherwise, object members are sorted by their names.

	static void formatString(const std::string& value, std::ostream& out);
		/// Formats the JSON string and streams it into ostream.
};


inline void Stringifier::condense(const Dynamic::Var& any, std::ostream& out)
{
	stringify(any, out, 0, -1, true);
}


inline void Stringifier::stringify(const Dynamic::Var& any, bool preserveInsertionOrder, std::ostream& out, unsigned int indent)
{
	stringify(any, out, indent, -1, preserveInsertionOrder);
}


}} // namespace Poco::JSON


#endif // JSON_JSONStringifier_INCLUDED
