//
// UpdateRequest.h
//
// $Id$
//
// Library: MongoDB
// Package: MongoDB
// Module:  UpdateRequest
//
// Definition of the UpdateRequest class.
//
// Copyright (c) 2012, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
//
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef MongoDB_UpdateRequest_INCLUDED
#define MongoDB_UpdateRequest_INCLUDED


#include "Poco/MongoDB/MongoDB.h"
#include "Poco/MongoDB/RequestMessage.h"
#include "Poco/MongoDB/Document.h"


namespace Poco {
namespace MongoDB {


class UpdateRequest : public RequestMessage
	/// Class for creating an OP_UPDATE client request. This request is used
	/// to update a document.
{
public:
	typedef enum
	{
		UPDATE_NOFLAGS = 0,
		// If set, the database will insert the supplied object into the
		// collection if no matching document is found.
		UPDATE_UPSERT = 1,
		// if set, the database will update all matching objects in the collection.
		// Otherwise only updates first matching doc.
		UPDATE_MULTIUPDATE = 2
	} Flags;

	UpdateRequest(const std::string& collectionName, Flags flags = UPDATE_NOFLAGS);
		/// Constructor.
		/// The full collection name is the concatenation of the database 
		/// name with the collection name, using a "." for the concatenation. For example, 
		/// for the database "foo" and the collection "bar", the full collection name is 
		/// "foo.bar".

	virtual ~UpdateRequest();
		/// Destructor
		

	Document& selector();
		/// Returns the selector document
		

	Document& update();
		/// The document to update

	Flags flags() const;
		/// Returns the flags

	void flags(Flags flags);
		/// Sets the flags

protected:
	void buildRequest(BinaryWriter& writer);


private:
	Flags       _flags;
	std::string _fullCollectionName;
	Document    _selector;
	Document    _update;
};


inline UpdateRequest::Flags UpdateRequest::flags() const
{
	return _flags;
}

inline void UpdateRequest::flags(UpdateRequest::Flags flags)
{
	_flags = flags;
}

inline Document& UpdateRequest::selector()
{
	return _selector;
}

inline Document& UpdateRequest::update()
{
	return _update;
}


} } // namespace Poco::MongoDB


#endif //MongoDB_UpdateRequest_INCLUDED
